@@ -1,17 +1,22 @@
 /* lock-test.c --- tests for the filesystem locking functions
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -19,6 +24,8 @@
 #include <apr_pools.h>
 #include <apr_time.h>
 
+#include "../svn_test.h"
+
 #include "svn_error.h"
 #include "svn_fs.h"
 
@@ -84,6 +91,40 @@ verify_matching_lock_paths(struct get_locks_baton_t *baton,
 }
 
 
+/* Create a filesystem in a directory called NAME, and populate it with
+ * the standard Greek tree.  Set *FS_P to the new filesystem object and
+ * *NEWREV_P to the head revision number.  Unwanted outputs may be NULL. */
+static svn_error_t *
+create_greek_fs(svn_fs_t **fs_p,
+                svn_revnum_t *newrev_p,
+                const char *name,
+                const svn_test_opts_t *opts,
+                apr_pool_t *pool)
+{
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root;
+  const char *conflict;
+  svn_revnum_t newrev;
+
+  /* Prepare a filesystem and a new txn. */
+  SVN_ERR(svn_test__create_fs(&fs, name, opts, pool));
+  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
+
+  /* Create the greek tree and commit it. */
+  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
+  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
+
+  if (fs_p)
+    *fs_p = fs;
+  if (newrev_p)
+    *newrev_p = newrev;
+  return SVN_NO_ERROR;
+}
+
+
 /*-----------------------------------------------------------------*/
 
 /** The actual lock-tests called by `make check` **/
@@ -92,33 +133,15 @@ verify_matching_lock_paths(struct get_locks_baton_t *baton,
 
 /* Test that we can create a lock--nothing more.  */
 static svn_error_t *
-lock_only(const char **msg,
-          svn_boolean_t msg_only,
-          svn_test_opts_t *opts,
+lock_only(const svn_test_opts_t *opts,
           apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *mylock;
 
-  *msg = "lock only";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-only",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-lock-only",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -138,33 +161,15 @@ lock_only(const char **msg,
 /* Test that we can create, fetch, and destroy a lock.  It exercises
    each of the five public fs locking functions.  */
 static svn_error_t *
-lookup_lock_by_path(const char **msg,
-                    svn_boolean_t msg_only,
-                    svn_test_opts_t *opts,
+lookup_lock_by_path(const svn_test_opts_t *opts,
                     apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *mylock, *somelock;
 
-  *msg = "lookup lock by path";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lookup-lock-by-path",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-lookup-lock-by-path",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -186,35 +191,17 @@ lookup_lock_by_path(const char **msg,
 /* Test that we can create a lock outside of the fs and attach it to a
    path.  */
 static svn_error_t *
-attach_lock(const char **msg,
-            svn_boolean_t msg_only,
-            svn_test_opts_t *opts,
+attach_lock(const svn_test_opts_t *opts,
             apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *somelock;
   svn_lock_t *mylock;
   const char *token;
 
-  *msg = "attach lock";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-attach-lock",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-attach-lock",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -245,35 +232,17 @@ attach_lock(const char **msg,
 
 /* Test that we can get all locks under a directory. */
 static svn_error_t *
-get_locks(const char **msg,
-          svn_boolean_t msg_only,
-          svn_test_opts_t *opts,
+get_locks(const svn_test_opts_t *opts,
           apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *mylock;
   struct get_locks_baton_t *get_locks_baton;
   apr_size_t i, num_expected_paths;
 
-  *msg = "get locks";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-get-locks",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-get-locks",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -375,6 +344,18 @@ get_locks(const char **msg,
                                        num_expected_paths, pool));
   }
 
+  /* A path that is longer and alphabetically earlier than some locked
+     paths, this exercises the r1205848 BDB lock code. */
+  {
+    static const char *expected_paths[] = { 0 };
+    num_expected_paths = 0;
+    get_locks_baton = make_get_locks_baton(pool);
+    SVN_ERR(svn_fs_get_locks(fs, "A/D/H/ABCDEFGHIJKLMNOPQR", get_locks_callback,
+                             get_locks_baton, pool));
+    SVN_ERR(verify_matching_lock_paths(get_locks_baton, expected_paths,
+                                       num_expected_paths, pool));
+  }
+
   return SVN_NO_ERROR;
 }
 
@@ -382,33 +363,15 @@ get_locks(const char **msg,
 /* Test that we can create, fetch, and destroy a lock.  It exercises
    each of the five public fs locking functions.  */
 static svn_error_t *
-basic_lock(const char **msg,
-           svn_boolean_t msg_only,
-           svn_test_opts_t *opts,
+basic_lock(const svn_test_opts_t *opts,
            apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *mylock, *somelock;
 
-  *msg = "basic locking";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-basic-lock",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-basic-lock",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -439,9 +402,7 @@ basic_lock(const char **msg,
 /* Test that locks are enforced -- specifically that both a username
    and token are required to make use of the lock.  */
 static svn_error_t *
-lock_credentials(const char **msg,
-                 svn_boolean_t msg_only,
-                 svn_test_opts_t *opts,
+lock_credentials(const svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   svn_fs_t *fs;
@@ -453,20 +414,8 @@ lock_credentials(const char **msg,
   svn_lock_t *mylock;
   svn_error_t *err;
 
-  *msg = "test that locking requires proper credentials";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-credentials",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, &newrev, "test-repo-lock-credentials",
+                          opts, pool));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -490,6 +439,7 @@ lock_credentials(const char **msg,
 
   /* Try to commit the file change.  Should fail, because we're nobody. */
   err = svn_fs_commit_txn(&conflict, &newrev, txn, pool);
+  SVN_TEST_ASSERT(! SVN_IS_VALID_REVNUM(newrev));
   if (! err)
     return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
@@ -502,6 +452,7 @@ lock_credentials(const char **msg,
 
   /* Try to commit the file change.  Should fail, because we're 'hortense'. */
   err = svn_fs_commit_txn(&conflict, &newrev, txn, pool);
+  SVN_TEST_ASSERT(! SVN_IS_VALID_REVNUM(newrev));
   if (! err)
     return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
@@ -514,6 +465,7 @@ lock_credentials(const char **msg,
 
   /* Try to commit the file change.  Should fail, because there's no token. */
   err = svn_fs_commit_txn(&conflict, &newrev, txn, pool);
+  SVN_TEST_ASSERT(! SVN_IS_VALID_REVNUM(newrev));
   if (! err)
     return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
@@ -525,6 +477,7 @@ lock_credentials(const char **msg,
 
   /* Commit should now succeed. */
   SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
 
   return SVN_NO_ERROR;
 }
@@ -536,9 +489,7 @@ lock_credentials(const char **msg,
    svn_fs_commit_txn().  Also, this test verifies that recursive
    lock-checks on directories is working properly. */
 static svn_error_t *
-final_lock_check(const char **msg,
-                 svn_boolean_t msg_only,
-                 svn_test_opts_t *opts,
+final_lock_check(const svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   svn_fs_t *fs;
@@ -550,20 +501,8 @@ final_lock_check(const char **msg,
   svn_lock_t *mylock;
   svn_error_t *err;
 
-  *msg = "test that locking is enforced in final commit step";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-final-lock-check",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, &newrev, "test-repo-final-lock-check",
+                          opts, pool));
 
   /* Make a new transaction and delete "/A" */
   SVN_ERR(svn_fs_begin_txn2(&txn, fs, newrev, SVN_FS_TXN_CHECK_LOCKS, pool));
@@ -582,6 +521,7 @@ final_lock_check(const char **msg,
   /* Try to commit the transaction.  Should fail, because a child of
      the deleted directory is locked by someone else. */
   err = svn_fs_commit_txn(&conflict, &newrev, txn, pool);
+  SVN_TEST_ASSERT(! SVN_IS_VALID_REVNUM(newrev));
   if (! err)
     return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
@@ -592,6 +532,7 @@ final_lock_check(const char **msg,
   SVN_ERR(svn_fs_set_access(fs, access));
   SVN_ERR(svn_fs_access_add_lock_token(access, mylock->token));
   SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
 
   return SVN_NO_ERROR;
 }
@@ -601,9 +542,7 @@ final_lock_check(const char **msg,
 /* If a directory's child is locked by someone else, we should still
    be able to commit a propchange on the directory. */
 static svn_error_t *
-lock_dir_propchange(const char **msg,
-                    svn_boolean_t msg_only,
-                    svn_test_opts_t *opts,
+lock_dir_propchange(const svn_test_opts_t *opts,
                     apr_pool_t *pool)
 {
   svn_fs_t *fs;
@@ -614,20 +553,8 @@ lock_dir_propchange(const char **msg,
   svn_fs_access_t *access;
   svn_lock_t *mylock;
 
-  *msg = "dir propchange can be committed with locked child";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-dir-propchange",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, &newrev, "test-repo-lock-dir-propchange",
+                          opts, pool));
 
   /* Become 'bubba' and lock "/A/D/G/rho". */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -648,154 +575,14 @@ lock_dir_propchange(const char **msg,
   /* Commit should succeed;  this means we're doing a non-recursive
      lock-check on directory, rather than a recursive one.  */
   SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
 
   return SVN_NO_ERROR;
 }
 
-
-
-/* DAV clients sometimes LOCK non-existent paths, as a way of
-   reserving names.  Check that this technique works. */
-static svn_error_t *
-lock_name_reservation(const char **msg,
-                      svn_boolean_t msg_only,
-                      svn_test_opts_t *opts,
-                      apr_pool_t *pool)
-{
-  svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root, *rev_root;
-  const char *conflict;
-  svn_revnum_t newrev;
-  svn_fs_access_t *access;
-  svn_lock_t *mylock;
-  svn_error_t *err;
-
-  *msg = "able to reserve a name (lock non-existent path)";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-name-reservation",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
-
-  /* Become 'bubba' and lock imaginary path  "/A/D/G2/blooga". */
-  SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
-  SVN_ERR(svn_fs_set_access(fs, access));
-  SVN_ERR(svn_fs_lock(&mylock, fs, "/A/D/G2/blooga", NULL, "", 0, 0,
-                      SVN_INVALID_REVNUM, FALSE, pool));
-
-  /* We are no longer 'bubba'.  We're nobody. */
-  SVN_ERR(svn_fs_set_access(fs, NULL));
-
-  /* Make a new transaction. */
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, newrev, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* This copy should fail, because an imaginary path in the target of
-     the copy is reserved by someone else. */
-  SVN_ERR(svn_fs_revision_root(&rev_root, fs, 1, pool));
-  err = svn_fs_copy(rev_root, "/A/D/G", txn_root, "/A/D/G2", pool);
-  if (! err)
-    return svn_error_create
-      (SVN_ERR_TEST_FAILED, NULL,
-       "Uhoh, copy succeeded when path within target was locked.");
-  svn_error_clear(err);
-
-  return SVN_NO_ERROR;
-}
-
-
-/* Test that we can set and get locks in and under a directory.  We'll
-   use non-existent FS paths for this test, though, as the FS API
-   currently disallows directory locking.  */
-static svn_error_t *
-directory_locks_kinda(const char **msg,
-                      svn_boolean_t msg_only,
-                      svn_test_opts_t *opts,
-                      apr_pool_t *pool)
-{
-  svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  svn_fs_access_t *access;
-  svn_lock_t *mylock;
-  apr_size_t num_expected_paths, i;
-  struct get_locks_baton_t *get_locks_baton;
-
-  *msg = "directory locks (kinda)";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-directory-locks-kinda",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* We are now 'bubba'. */
-  SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
-  SVN_ERR(svn_fs_set_access(fs, access));
-
-  /*** Lock some various, non-existent, yet dir-name-spacily
-       overlapping paths; verify. ***/
-  {
-    static const char *expected_paths[] = {
-      "/Program Files/Tigris.org/Subversion",
-      "/Program Files/Tigris.org",
-      "/Stuff/Junk/Fluff",
-      "/Program Files",
-    };
-    num_expected_paths = sizeof(expected_paths) / sizeof(const char *);
-
-    /* Lock all paths under /A/D/G. */
-    for (i = 0; i < num_expected_paths; i++)
-      {
-        SVN_ERR(svn_fs_lock(&mylock, fs, expected_paths[i], NULL, "", 0, 0,
-                            SVN_INVALID_REVNUM, FALSE, pool));
-      }
-    get_locks_baton = make_get_locks_baton(pool);
-    SVN_ERR(svn_fs_get_locks(fs, "/", get_locks_callback,
-                             get_locks_baton, pool));
-    SVN_ERR(verify_matching_lock_paths(get_locks_baton, expected_paths,
-                                       num_expected_paths, pool));
-  }
-
-  /*** Now unlock a "middle directory" ***/
-  {
-    static const char *expected_paths[] = {
-      "/Program Files/Tigris.org/Subversion",
-      "/Stuff/Junk/Fluff",
-      "/Program Files",
-    };
-    num_expected_paths = sizeof(expected_paths) / sizeof(const char *);
-
-    SVN_ERR(svn_fs_unlock(fs, "/Program Files/Tigris.org", NULL,
-                          TRUE, pool));
-    get_locks_baton = make_get_locks_baton(pool);
-    SVN_ERR(svn_fs_get_locks(fs, "/", get_locks_callback,
-                             get_locks_baton, pool));
-    SVN_ERR(verify_matching_lock_paths(get_locks_baton, expected_paths,
-                                       num_expected_paths, pool));
-  }
-
-  return SVN_NO_ERROR;
-}
-
-
 /* Test that locks auto-expire correctly. */
 static svn_error_t *
-lock_expiration(const char **msg,
-                svn_boolean_t msg_only,
-                svn_test_opts_t *opts,
+lock_expiration(const svn_test_opts_t *opts,
                 apr_pool_t *pool)
 {
   svn_fs_t *fs;
@@ -808,20 +595,8 @@ lock_expiration(const char **msg,
   svn_error_t *err;
   struct get_locks_baton_t *get_locks_baton;
 
-  *msg = "test that locks can expire";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-expiration",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, &newrev, "test-repo-lock-expiration",
+                          opts, pool));
 
   /* Make a new transaction and change rho. */
   SVN_ERR(svn_fs_begin_txn2(&txn, fs, newrev, SVN_FS_TXN_CHECK_LOCKS, pool));
@@ -844,6 +619,7 @@ lock_expiration(const char **msg,
   /* Try to commit.  Should fail because we're 'nobody', and the lock
      hasn't expired yet. */
   err = svn_fs_commit_txn(&conflict, &newrev, txn, pool);
+  SVN_TEST_ASSERT(! SVN_IS_VALID_REVNUM(newrev));
   if (! err)
     return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
@@ -880,39 +656,22 @@ lock_expiration(const char **msg,
   }
 
   SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
 
   return SVN_NO_ERROR;
 }
 
 /* Test that a lock can be broken, stolen, or refreshed */
 static svn_error_t *
-lock_break_steal_refresh(const char **msg,
-                         svn_boolean_t msg_only,
-                         svn_test_opts_t *opts,
+lock_break_steal_refresh(const svn_test_opts_t *opts,
                          apr_pool_t *pool)
 {
   svn_fs_t *fs;
-  svn_fs_txn_t *txn;
-  svn_fs_root_t *txn_root;
-  const char *conflict;
-  svn_revnum_t newrev;
   svn_fs_access_t *access;
   svn_lock_t *mylock, *somelock;
 
-  *msg = "breaking, stealing, refreshing a lock";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-steal-refresh",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, NULL, "test-repo-steal-refresh",
+                          opts, pool));
 
   /* Become 'bubba' and lock "/A/D/G/rho". */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -972,9 +731,7 @@ lock_break_steal_refresh(const char **msg,
 /* Test that svn_fs_lock() and svn_fs_attach_lock() can do
    out-of-dateness checks..  */
 static svn_error_t *
-lock_out_of_date(const char **msg,
-                 svn_boolean_t msg_only,
-                 svn_test_opts_t *opts,
+lock_out_of_date(const svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   svn_fs_t *fs;
@@ -986,20 +743,8 @@ lock_out_of_date(const char **msg,
   svn_lock_t *mylock;
   svn_error_t *err;
 
-  *msg = "check out-of-dateness before locking";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
-  /* Prepare a filesystem and a new txn. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-lock-out-of-date",
-                              opts->fs_type, pool));
-  SVN_ERR(svn_fs_begin_txn2(&txn, fs, 0, SVN_FS_TXN_CHECK_LOCKS, pool));
-  SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-
-  /* Create the greek tree and commit it. */
-  SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
-  SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_ERR(create_greek_fs(&fs, &newrev, "test-repo-lock-out-of-date",
+                          opts, pool));
 
   /* Commit a small change to /A/D/G/rho, creating revision 2. */
   SVN_ERR(svn_fs_begin_txn2(&txn, fs, newrev, SVN_FS_TXN_CHECK_LOCKS, pool));
@@ -1007,6 +752,7 @@ lock_out_of_date(const char **msg,
   SVN_ERR(svn_test__set_file_contents(txn_root, "/A/D/G/rho",
                                       "new contents", pool));
   SVN_ERR(svn_fs_commit_txn(&conflict, &newrev, txn, pool));
+  SVN_TEST_ASSERT(SVN_IS_VALID_REVNUM(newrev));
 
   /* We are now 'bubba'. */
   SVN_ERR(svn_fs_create_access(&access, "bubba", pool));
@@ -1049,18 +795,27 @@ lock_out_of_date(const char **msg,
 struct svn_test_descriptor_t test_funcs[] =
   {
     SVN_TEST_NULL,
-    SVN_TEST_PASS(lock_only),
-    SVN_TEST_PASS(lookup_lock_by_path),
-    SVN_TEST_PASS(attach_lock),
-    SVN_TEST_PASS(get_locks),
-    SVN_TEST_PASS(basic_lock),
-    SVN_TEST_PASS(lock_credentials),
-    SVN_TEST_PASS(final_lock_check),
-    SVN_TEST_PASS(lock_dir_propchange),
-    SVN_TEST_XFAIL(lock_name_reservation),
-    SVN_TEST_XFAIL(directory_locks_kinda),
-    SVN_TEST_PASS(lock_expiration),
-    SVN_TEST_PASS(lock_break_steal_refresh),
-    SVN_TEST_PASS(lock_out_of_date),
+    SVN_TEST_OPTS_PASS(lock_only,
+                       "lock only"),
+    SVN_TEST_OPTS_PASS(lookup_lock_by_path,
+                       "lookup lock by path"),
+    SVN_TEST_OPTS_PASS(attach_lock,
+                       "attach lock"),
+    SVN_TEST_OPTS_PASS(get_locks,
+                       "get locks"),
+    SVN_TEST_OPTS_PASS(basic_lock,
+                       "basic locking"),
+    SVN_TEST_OPTS_PASS(lock_credentials,
+                       "test that locking requires proper credentials"),
+    SVN_TEST_OPTS_PASS(final_lock_check,
+                       "test that locking is enforced in final commit step"),
+    SVN_TEST_OPTS_PASS(lock_dir_propchange,
+                       "dir propchange can be committed with locked child"),
+    SVN_TEST_OPTS_PASS(lock_expiration,
+                       "test that locks can expire"),
+    SVN_TEST_OPTS_PASS(lock_break_steal_refresh,
+                       "breaking, stealing, refreshing a lock"),
+    SVN_TEST_OPTS_PASS(lock_out_of_date,
+                       "check out-of-dateness before locking"),
     SVN_TEST_NULL
   };
